import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, ParamMap, Router } from '@angular/router';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { filter, switchMap } from 'rxjs/operators';
import { Format } from 'src/app/models/format.model';
import { FormatService } from 'src/app/services';

@Component({
  selector: 'app-format-form',
  templateUrl: './format-form.component.html',
  styleUrls: ['./format-form.component.scss'],
})
export class FormatFormComponent implements OnInit {

  format: Format = new Format();
  form: FormGroup;
  title: string;

  constructor(
    private _formatService: FormatService,
    private _route: ActivatedRoute,
    private _router: Router,
    private _fb: FormBuilder,
  ) {
  }

  ngOnInit() {
    this.title = this._route.snapshot.data.title;
    this.initForm();

    this._route.paramMap.pipe(
      filter((paramMap: ParamMap) => (paramMap.get('id') !== null)),
      switchMap((paramMap: ParamMap) => this._formatService.findById(paramMap.get('id'))))
      .subscribe((format: Format) => {

        this.format = format;

        this.initForm();

      });
  }

  initForm() {
    this.form = this._fb.group({
      id: [this.format.id],
      name: [this.format.name, Validators.required],
      mapServerType: [this.format.mapServerType, Validators.required],
      fileExtension: [this.format.fileExtension, Validators.required],
    });
  }

  onSubmit() {
    if (!this.formInvalid) {
      this.format = new Format(this.form.value);
      this._formatService.replaceOrCreate(this.format)
        .subscribe(
          (formatCreated) => {
            this._router.navigate(['/formats', formatCreated.id]);
          },
          (err) => {
            alert(err.message);
          },
        );
    }
  }

  // Getters for each property
  get name() {
    return this.form.controls['name'];
  }

  get mapServerType() {
    return this.form.controls['mapServerType'];
  }

  get fileExtension() {
    return this.form.controls['fileExtension'];
  }

  get formInvalid() {
    return this.form.invalid;
  }
}
